/*****************************************************************************
*   $Id: sbasic.c, v8.0 2000/08/08 00:08:00 Thierry Godefroy $
*
*   Copyright (c) 2000, Thierry Godefroy & Darren Hiebert
*
*   This source code is released for free distribution under the terms of the
*   GNU General Public License.
*
*   This module contains functions for generating tags for S(uper)BASIC
*   programs (SuperBASIC is the QDOS systems BASIC interpreter, SBASIC is the
*   SMSQ(/E) systems BASIC semi-compiler).
*****************************************************************************/

/*============================================================================
=   Include files
============================================================================*/
#include "general.h"	/* must always come first */

#include <ctype.h>	/* to define isalpha(), isalnum(), isspace() */
#include <string.h>

#include "entry.h"
#include "main.h"
#include "parse.h"
#include "read.h"
#include "vstring.h"

static void makeProcedureTag(const vString * const name, int start)
{
    tagEntryInfo e;

    initTagEntry(&e, vStringValue(name));

    e.kindName = "procedure";
    e.kind     = 'p';
    /* Do not record the PROCedure parameters list into the tag file */
    e.truncateLine = TRUE;
    /* Skip any BASIC line number */
    e.filePosition += start;
    
    makeTagEntry(&e);
}

static void makeFunctionTag(const vString * const name, int start)
{
    tagEntryInfo e;

    initTagEntry(&e, vStringValue(name));

    e.kindName = "function";
    e.kind     = 'f';
    /* Do not record the FuNction parameters list into the tag file */
    e.truncateLine = TRUE;
    /* Skip any BASIC line number */
    e.filePosition += start;

    makeTagEntry(&e);
}

extern void createSbasicTags(void)
{
    vString *vLine = vStringNew();
    vString *name = vStringNew();
    const char *line;
    char *cp;
    int offset;
    int start;

    while ((line = fileReadLine(vLine)) != NULL) {

        /* Check for a possible "DEFine" keyword in the line */
    	if ((offset = strfnd("DEFine", line)) >= 0) {
    	    cp = (char *)line;

	    /* Skip any BASIC line number and remember the actual
             * start of the line.
	     */
    	    start = 0;
    	    while (!isalpha(*(cp++)))
    	    	start++;

    	    /* Point after the "DEFine" keyword */
	    cp = (char *)line + offset + 6;

	    /* Check for either "PROCedure" or "FuNction" following it */
    	    if ((offset = strfnd("PROCedure", cp)) >= 0) {
    	    	/* Point after the "PROCedure" keyword */
    	    	cp = cp + offset + 9;
    	    	/* Skip spacing characters */
		while (isspace(*cp))
		    cp++;

		/* Get the PROCedure name... */
		while (isalnum((int)*cp)  ||  *cp == '_') {
		    vStringPut(name, (int)*cp);
		    ++cp;
		}
		vStringTerminate(name);
		/* and register it as a procedure tag. */
		makeProcedureTag(name, start);
		vStringClear(name);
		
    	    } else if ((offset = strfnd("FuNction", cp)) >= 0) {
    	    	/* Point after the "FuNction" keyword */
    	    	cp = cp + offset + 8;
    	    	/* Skip spacing characters */
		while (isspace(*cp))
		    cp++;

		/* Get the FuNction name... */
		while (isalnum((int)*cp)  ||  *cp == '_') {
		    vStringPut(name, (int)*cp);
		    ++cp;
		}
		vStringTerminate(name);
		/* and register it as a function tag. */
    	    	makeFunctionTag(name, start);
		vStringClear(name);
		
    	    }
    	}
    }
    vStringDelete(name);
    vStringDelete(vLine);
}
